/*
 * Copyright (c) 2016-2017 Illumon and Patent Pending
 */

#pragma once

#include "BinaryStoreWriter.hpp"

/*
 * This is an example wrapper for logging market trades to an Iris binary store.
 *
 * The resultant output file, can then be streamed by a log tailer to the data
 * import server; which makes it available for use in queries.
 *
 * Wrappers make it more convenient to write data to your binary store, as
 * otherwise you must call writeRow with a sequence of unnamed arguments, of
 * the right type.  Because the writeRow call uses variable template arguments,
 * it will compile even if you get the types wrong; but will fail at runtime
 * with a logic_error.
 */
class TradeWriter
  : illumon::binarystore::BinaryStoreWriter
{
public:
  /* The constructor initializes the fields that will be written.
   *
   * The field declarations must match the schema expected by the data import server. */
  TradeWriter();

  /*
   * Write a trade entry row to the binary store.
   *
   * Timestamps are represented as nano-seconds from the unix epoch.
   */
  void writeTrade(uint64_t insertTime, std::string const & usym, std::string const & sym, std::string const & side, double price, int size, uint64_t exchangeTimestamp, std::string const & exchange);

  /* Pass through the open and close calls. */
  using illumon::binarystore::BinaryStoreWriter::open;
  using illumon::binarystore::BinaryStoreWriter::close;
};
