/*
 * Copyright (c) 2016-2017 Illumon and Patent Pending
 */

#pragma once

#include <noncopyable.hpp>

namespace illumon {

namespace util {

class FileDescriptor
  : noncopyable
{
  public:
    // create a file descriptor that doesn't have anything open
    FileDescriptor();

    // create a file descriptor to manage the underlying (valid) fd, which is passed in.
    FileDescriptor(int fd);

    // close the existing managed fd, replacing it with the new one
    void reset(int fd=-1);

    // implicit conversion to int so that FileDescriptor can be used with system calls
    operator int() const;

    // get the integer version of the file descriptor
    int get() const;

    // returns true if the file is open (in reality if the fd is not less than 0)
    bool isOpen() const;

    // close the file descriptor, does not throw on error, simply emits a warning
    // message.  To get a throw, use an explicit reset.
    ~FileDescriptor();

  private:
    int m_rawFd;
};

}}; // namespace illumon::util
