/*
 * Copyright (c) 2016-2017 Illumon and Patent Pending
 */

#include "FileDescriptor.hpp"
#include "ErrnoException.hpp"

#include <cerrno>
#include <string>
#include <stdexcept>

#include <unistd.h>

namespace illumon {

namespace util {

FileDescriptor::FileDescriptor()
  : m_rawFd(-1)
{
}

FileDescriptor::FileDescriptor(int fd)
  : m_rawFd(fd)
{
  if (fd < 0)
  {
    throw std::runtime_error("File descriptors must be non-negative in initialization.");
  }
}

void FileDescriptor::reset(int fd)
{
  // we set the fd to the new fd even if the close fails, so we at least
  // won't be leaking the newly passed in file descriptor during the likely
  // stack unwind.
  int oldFd = m_rawFd;
  m_rawFd = fd;

  if (oldFd >= 0)
  {
    errno = 0;
    int res = ::close(oldFd);
    if (res < 0)
    {
      throw errno_exception("Could not close file descriptor: ", errno);
    }
  }
}

FileDescriptor::operator int() const
{
  return m_rawFd;
}

int FileDescriptor::get() const
{
  return m_rawFd;
}

bool FileDescriptor::isOpen() const
{
  return m_rawFd >= 0;
}

FileDescriptor::~FileDescriptor()
{
  reset();
}

}}; // namespace illumon::util
