#include <iostream>

#include "BinaryStoreWriter.hpp"

using std::cerr;
using std::endl;
using namespace illumon::binarystore;

int main(int argc, char *argv[])
{
  BinaryStoreWriter writer;

  writer.addColumn("i8"     , SupportedType::Byte);
  writer.addColumn("i16"    , SupportedType::Short);
  writer.addColumn("i32"    , SupportedType::Int);
  writer.addColumn("i64"    , SupportedType::Long);
  writer.addColumn("Float"  , SupportedType::Float);
  writer.addColumn("Double" , SupportedType::Double);
  writer.addColumn("Char"   , SupportedType::Char);
  writer.addColumn("Bool"   , SupportedType::Boolean);
  writer.addColumn("Stringy", SupportedType::String);

  writer.open("writer.test.bin");

  writer.writeRow((int8_t)(NULL_BYTE + 1), (int16_t) (NULL_SHORT + 1), (int32_t)(NULL_INT + 1),
                      (int64_t)(NULL_LONG+1), NULL_FLOAT + 1.0f, NULL_DOUBLE + 1.0f,
                      (char) (1), false, "A");

  std::string potato = "a";
  
  int i = 0;
  for (i = 0; i < 100; i++) {
      writer.writeRow((int8_t) i, (int16_t) (i << 8), (int32_t)(i * 2), (int64_t)(-i * 8), 1.42f * i, 4.98 * i, (char) ('a' + i), (i % 2) == 0, potato);
      potato += (char)( (i % 31) + 'a');
  }
   
  //Now test some CPP specific cases
  i++;
  writer.writeRow((int8_t)i, (int16_t) (i << 8), (int32_t)(i * 2), (int64_t)(-i * 8), 1.42f * i, 4.98 * i, (char) ('a' + i), (i % 2) == 0, static_cast<char *>(nullptr));
  i++;
  writer.writeRow((int8_t)i, (int16_t) (i << 8), (int32_t)(i * 2), (int64_t)(-i * 8), 1.42f * i, 4.98 * i, (char) ('a' + i), (i % 2) == 0, std::string("Quux"));
  i++;
  writer.writeRow((int8_t)i, (int16_t) (i << 8), (int32_t)(i * 2), (int64_t)(-i * 8), 1.42f * i, 4.98 * i, (char) ('a' + i), (i % 2) == 0, std::string());
  i++;
  writer.writeRow((int8_t)i, (int16_t) (i << 8), (int32_t)(i * 2), (int64_t)(-i * 8), 1.42f * i, 4.98 * i, (char) ('a' + i), (i % 2) == 0, "");
  i++;
  writer.writeRow((int8_t)i, (int16_t) (i << 8), (int32_t)(i * 2), (int64_t)(-i * 8), 1.42f * i, 4.98 * i, (char) ('a' + i), (i % 2) == 0, boost::none);

  boost::optional<std::string> optional1;
  boost::optional<std::string> optional2(std::string("Foo"));
  std::string bar("Bar");
  boost::optional<std::string const &> optional3(bar);
  boost::optional<std::string const &> optional4;
  boost::optional<std::string &> optional5(bar);
  boost::optional<std::string &> optional6;
  
  i++;
  writer.writeRow((int8_t)i, (int16_t) (i << 8), (int32_t)(i * 2), (int64_t)(-i * 8), 1.42f * i, 4.98 * i, (char) ('a' + i), (i % 2) == 0, optional1);
  i++;
  writer.writeRow((int8_t)i, (int16_t) (i << 8), (int32_t)(i * 2), (int64_t)(-i * 8), 1.42f * i, 4.98 * i, (char) ('a' + i), (i % 2) == 0, optional2);
  i++;
  writer.writeRow((int8_t)i, (int16_t) (i << 8), (int32_t)(i * 2), (int64_t)(-i * 8), 1.42f * i, 4.98 * i, (char) ('a' + i), (i % 2) == 0, optional3);
  i++;
  writer.writeRow((int8_t)i, (int16_t) (i << 8), (int32_t)(i * 2), (int64_t)(-i * 8), 1.42f * i, 4.98 * i, (char) ('a' + i), (i % 2) == 0, optional4);
  i++;
  writer.writeRow((int8_t)i, (int16_t) (i << 8), (int32_t)(i * 2), (int64_t)(-i * 8), 1.42f * i, 4.98 * i, (char) ('a' + i), (i % 2) == 0, optional5);
  i++;
  writer.writeRow((int8_t)i, (int16_t) (i << 8), (int32_t)(i * 2), (int64_t)(-i * 8), 1.42f * i, 4.98 * i, (char) ('a' + i), (i % 2) == 0, optional6);

  i++;
  writer.writeRow((int8_t)i, (int16_t) (i << 8), (int32_t)(i * 2), 3L, 1.42f * i, 4.98 * i, (char) ('a' + i), (i % 2) == 0, "Hey");
  i++;
  writer.writeRow((int8_t)i, (int16_t) (i << 8), (int32_t)(i * 2), boost::none, 1.42f * i, 4.98 * i, (char) ('a' + i), (i % 2) == 0, "Hey");
  i++;
  writer.writeRow((int8_t)i, (int16_t) (i << 8), (int32_t)(i * 2), boost::optional<long>(4L),1.42f * i, 4.98 * i, (char) ('a' + i), (i % 2) == 0, "Hey");
  i++;
  writer.writeRow((int8_t)i, (int16_t) (i << 8), (int32_t)(i * 2), boost::optional<long>(),1.42f * i, 4.98 * i, (char) ('a' + i), (i % 2) == 0, "Hey");
  i++;
  writer.writeRow((int8_t)i, (int16_t) (i << 8), (int32_t)(i * 2), 1L, 1.42f * i, boost::optional<double>(), (char) ('a' + i), (i % 2) == 0, "Hey");
  i++;
  writer.writeRow((int8_t)i, (int16_t) (i << 8), (int32_t)(i * 2), 1L, boost::optional<float>(), 4.98 * i, (char) ('a' + i), (i % 2) == 0, "Hey");
  i++;
  writer.writeRow(boost::optional<int8_t>(), (int16_t) (i << 8), (int32_t)(i * 2), 1L, 1.42f * i, 4.98*i, (char) ('a' + i), (i % 2) == 0, "Hey");
  i++;
  writer.writeRow((int8_t)i, boost::optional<int16_t>(), (int32_t)(i * 2), 1L, 1.42f * i, 4.98*i, (char) ('a' + i), (i % 2) == 0, "Hey");
  i++;
  writer.writeRow((int8_t)i,(int16_t)(i<<8), boost::optional<int32_t>(), 1L, 1.42f * i, 4.98*i, (char) ('a' + i), (i % 2) == 0, "Hey");

  writer.close();

  exit(0);
}
